;$Id: rasdq.pro,v 1.1 2002/04/23 14:47:01 jmspark Exp $



pro weedoub,xin,fxin,dfxin,dx,xn,fxn,dfxn,srt=srt
;****************************************************************
;*								*
;* input array of measurements of a function of one variable 	*
;* with error bars. returns a sorted version of array with   	*
;* all points measured within dx of one another averaged	*
;* statistically correctly					*
;*								*
;* Input :							*
;*	x	:	Array of unsorted values of free	*
;*			variable				*
;*	fx	:	measured values of function at those	*
;*			points					*
;*	dfx	:	error bars on those measurements	*
;*	dx	:	tolerance in x beyond which fx are	*
;*			considered different			*
;*	srt	:	if present assume x is sorted on input	*
;*	gauss	:	by default poisson statistics is used :	*
;*								*
;*				average=sum/N			*
;*				error=sqrt(sum(error^2))/N	*
;*								*
;*			if keyword normd is present and not 0	*
;*			gaussian distribution is assumed :	*
;*								*
;*				average=sum(y/dy^2)/sum(1/dy^2)	*
;*				error=1/sqrt(sum(1/dy^2))	*
;*								*
;* Output :							*
;*	xn	:	sorted values of free variable		*
;*	fxn	:	corresponding best estimates of		*
;*			function				*
;*	dfxn	:	error bars on fxn			*
;*								*
;****************************************************************
nx	=	n_elements(xin)

;--------------------------------------------------
;| copy input to other array so as not to destroy |
;--------------------------------------------------
x	=	fltarr(nx)
fx	=	fltarr(nx)
dfx	=	fltarr(nx)
x	=	xin
fx	=	fxin
dfx	=	dfxin

xn	=	fltarr(nx)
fxn	=	fltarr(nx)
dfxn	=	fltarr(nx)


if not keyword_set(srt) then ix=sort(x) else ix=findgen(nx)

;----------------------
;| Poisson Statistics |
;----------------------
in	=	-1
j	=	-1
for i=0,nx-1 do begin				$
 if i gt j then begin				$
  fxav	=	0
  ins2	=	0
  xtot	=	0
  nav	=	0
  for j=i,nx-1 do begin				$
   if abs(x(ix(i))-x(ix(j))) gt dx then goto,different
    xtot	=	xtot+x(ix(j))
    fxav	=	fxav+fx(ix(j))
    ins2	=	ins2+dfx(ix(j))^2
    nav		=	nav+1
  endfor
  different:
  j=j-1
  in	=	in+1
  xn(in)	=	xtot/nav
  fxn(in)	=	fxav/nav
  dfxn(in)	=	sqrt(ins2)/nav
 endif
endfor


xn	=	xn(0:in)
fxn	=	fxn(0:in)
dfxn	=	dfxn(0:in)

end

pro avy,x,y,z,dz,dx,xo,zo,dzo,yran=yran

if n_elements(yran) ne 2 then yran=[min(y),max(y)]
n	=	n_elements(x)

lis	=	where( (y ge yran(0) and y le yran(1)) and z gt -1e30,no)
if no eq 0 then print,'from avy.pro, no data points'
xo	=	x(lis)
zo	=	z(lis)
dzo	=	dz(lis)

weedoub,xo,zo,dzo,dx,xo,zo,dzo

end



function flunct,arry
;________________________________________________________________
;|								|
;| function to calculate the delta value, which is sqrt of 	|
;| ( square's average - average's square)/average's square	|
;|______________________________________________________________|
n	=	n_elements(arry)
ave	=	total(arry)/n
if abs(ave) lt 1.e-3 then return,0
aves	=	total(arry^2)/n
return,sqrt(abs(aves-ave^2)/ave^2)
end

pro sxyar,data,in
;________________________________________________________________
;|								|
;| Selecting the x and y axis from the data structure.		|
;| subroutine of the sxya.pro					|
;| 								|
;| input :							|
;|	-- data,  data array in the structure,assumining that	|
;|	          data(*,1) & data(*,2) are the			|
;|		  intensity and corresponding error bars	|
;| output:							|
;|	-- in, 	  the index 		 			|
;|		  data(*,in) has the maximum deviation		|
;|		  if the deviation of data(*,0) is zero		|
;|______________________________________________________________|

nde		=	n_elements(data(0,*))
delt		=	fltarr(nde)
for i=0,nde-1 do delt(i)	=	flunct(data(*,i))

delt(1)	=	-1.e32
delt(2)	=	-1.e32
lis	=	sort(delt)
in	=	lis(nde-1)

end


pro sxya,datas
;________________________________________________________________
;|								|
;| Selecting the x and y axis from the data structure.		|
;| subroutine of the rnistq.pro					|
;| Differing from saxis.pro this routine is to select the 	|
;| horizontal variable with reference to the standard deviation	|
;| If the standard deviation of datas.de(*,0) is no zero, 	|
;| nothing will be changed					|
;| 								|
;| Input :							|
;|	- datas, the data structure.				|
;|								|
;| Output :							|
;|	- datas, with right x axis				|
;|______________________________________________________________|

dedd				=	fltarr(datas.ndp,datas.nde)
dedd				=	datas.de(*,*)
	sxyar,dedd,inn
	tde0			=	datas.tde(0)
	ude0			=	datas.ude(0)
	datas.de(*,0)		=	dedd(*,inn)
	datas.de(*,inn)		=	dedd(*,0)
	datas.tde(0)		=	datas.tde(inn)
	datas.ude(0)		=	datas.ude(inn)
	datas.tde(inn)		=	tde0
	datas.ude(inn)		=	ude0

end

pro roneasdq,file_name,data,gaussan=gaussan,xtitle=xtt,ytitle=ytt
;--------------------------------------------------------------------
;|								    |
;| Purpose	: Read ASD data file of the format which is produced|
;|		  by the NIST neutron spectrometer controle program |
;|		  ICP  const-Q buffer				    |
;|								    |
;|								    |
;| Input	: file-name, the file to be read		    |
;| Output	: data, the data structure			    |
;| externals	: gendast, to generate data structure		    |
;|								    |
;--------------------------------------------------------------------

in	=	1000
junk		=	''
efixt		=	''
fnam		=	''
date		=	''
scanty		=	''
dataty		=	''
tit		=	''
dede		=	fltarr(in,9)
dqc		=	fltarr(3)
ddq		=	dqc


openr,lun,file_name,/get_lun

;----------------------------------
;| read in header data for q-file |
;----------------------------------
readf,lun,format='(x,a12,3x,a17,3x,a1,5x,f8.0,f7.0,1x,a4,2x,i4,3x,a3,2x)',$
      fnam,date,scanty,r0,r1,scanty,npt,dataty
mon	=	r0*r1

readf,lun,junk
readf,lun,format='(a100)',tit
readf,lun,format='(4i3,5x,3i3,7x,9x,f8.3)', $
      col0,col1,col2,col3,msm,mss,msa,dangle
readf,lun,junk
readf,lun,aa,bb,cc,alfa,beta,gama
readf,lun,junk
readf,lun,dec,dde,def,dmdsp,dadsp,dtmpst,tinc
readf,lun,format='(27x,a8)',efixt
readf,lun,dqc,ddq,dhfld
readf,lun,junk
readf,lun,format='(x,a12,8x,i4)',junk,npix
readf,lun,junk

; asd data format
tempasd	=	fltarr(npix)
asd	=	fltarr(in,npix)

;Count the number of columns to be read. Assume standard set of columns.
iw	=	0
while ( readword(junk,iw) ne '') do iw	=	iw+1

dqc0	=	dqc(0)		&	ddq0	=	ddq(0)
dqc1	=	dqc(1)		&	ddq1	=	ddq(1)
dqc2	=	dqc(2)		&	ddq2	=	ddq(2)

taum	=	2.*!pi/dmdsp
taua	=	2.*!pi/dadsp

;-----------------------------
;| start reading actual data |
;-----------------------------
iu	=	0
on_ioerror, endwhil
while not eof(lun) do begin
	if iu ge in then stop,'more than 1000 data-points'
	; no temperature record
	tt	=	0.
	hf	=	0.
	if iw eq 6 then 					$
		readf,lun,qxt,qyt,qzt,et,mint,it		$
	else begin
		if iw eq 7 then begin
		  if readword(junk,4) eq 'T-act' then  		$				$
		    ; w/ T record
		    readf,lun,qxt,qyt,qzt,et,tt,mint,it 	$
		  else if readword(junk,4) eq 'H-Field' then  	$				$
		    ; w/ H record
		    readf,lun,qxt,qyt,qzt,et,hf,mint,it
		end else readf,lun,qxt,qyt,qzt,et,tt,hf,mint,it
	end
	dede(iu,0)	=	et
	dede(iu,1)	=	it
	if keyword_set(gaussan) then dede(iu,2)	= gauss(it) else $
	dede(iu,2)	=	sqrt(it)
	dede(iu,3)	=	qxt
	dede(iu,4)	=	qyt
	dede(iu,5)	=	qzt
	dede(iu,6)	=	tt
	dede(iu,7)	=	hf
	dede(iu,8)	=	mint

	; asd data
	readf,lun,tempasd
	asd(iu,*)	=	tempasd
	iu	=	iu+1
endwhil:
endwhile
free_lun,lun
dede	=	dede(0:iu-1,*)
asd	=	asd(0:iu-1,*)

;----------------------------------------
;| generating the dat structure		|
;----------------------------------------
gendast,data,3,26,9,iu,npix,/asd

if iu ne npt then begin
	print,'(FROM rnistq.pro) # of data points is different ' $
		+'from the # shown on top'
	print,'                  of the file. Is this an aborted file?'
endif


data.inorm	=	0
inorm		=	data.inorm
data.tnvh(inorm)=	'monitor'
data.tnvh(1)	=	'RM collim'
data.tnvh(2)	=	'MS collim'
data.tnvh(3)	=	'SA collim'
data.tnvh(4)	=	'AD collim'
data.tnvh(5)	=	efixt
data.tnvh(6)	=	'hw center'
data.tnvh(7)	=	'hc'
data.tnvh(8)	=	'kc'
data.tnvh(9)	=	'H field'
data.tnvh(10)	=	'mon mosaic'
data.tnvh(11)	=	'samp mosaic'
data.tnvh(12)	=	'ana mosaic'
data.tnvh(13)	=	'tau monochromator'
data.tnvh(14)	=	'tau analyzer'
data.tnvh(15)	=	'hw step'
data.tnvh(16)	=	'dh'
data.tnvh(17)	=	'dk'
data.tnvh(18)	=	'dl'
data.tnvh(19)	=	'phi'
data.tnvh(20)	=	'a'
data.tnvh(21)	=	'b'
data.tnvh(22)	=	'c'
data.tnvh(23)	=	'alfa'
data.tnvh(24)	=	'beta'
data.tnvh(25)	=	'gamma'

data.unvh(inorm)	=	'counts'
data.unvh(1)		=	'min'
data.unvh(2)		=	'min'
data.unvh(3)		=	'min'
data.unvh(4)		=	'min'
data.unvh(5)		=	'meV'
data.unvh(6)		=	'mev'
data.unvh(7)		=	'rlu'
data.unvh(8)		=	'rlu'
data.unvh(9)		=	'Tesla'
data.unvh(10)		=	'min'
data.unvh(11)		=	'min'
data.unvh(12)		=	'min'
data.unvh(13)		=	'AA^-1'
data.unvh(14)		=	'AA^-1'
data.unvh(15)		=	'mev'
data.unvh(16)		=	'rlu'
data.unvh(17)		=	'rlu'
data.unvh(18)		=	'rlu'
data.unvh(19)	=	'deg'
data.unvh(20)	=	'AA'
data.unvh(21)	=	'AA'
data.unvh(22)	=	'AA'
data.unvh(23)	=	'deg'
data.unvh(24)	=	'deg'
data.unvh(25)	=	'deg'

data.tsvh(0)	=	'title'
data.tsvh(1)	=	'filename'
data.tsvh(2)	=	'date'

data.tde(0)		=	'E'
data.tde(1)		=	'intensity'
data.tde(2)		=	'error'
data.tde(3)		=	'h'
data.tde(4)		=	'k'
data.tde(5)		=	'l'
data.tde(6)		=	'T'
data.tde(7)		=	'Hfield'
data.tde(8)		=	'time'

data.ude(0)		=	'meV'
data.ude(1)		=	'counts'
data.ude(2)		=	'counts'
data.ude(3)		=	'rlu'
data.ude(4)		=	'rlu'
data.ude(5)		=	'rlu'
data.ude(6)		=	'K'
data.ude(7)		=	'Tesla'
data.ude(8)		=	'min'



;_________________________________________________
;| transfer header data to output data-structure |
;|_______________________________________________|
data.svh(0)	=	tit
data.svh(1)	=	fnam
data.svh(2)	=	date
data.scan	=	scanty


if scanty eq 'TIME' then begin
	data.tnvh(inorm)	=	'time'
	data.unvh(inorm)	=	'seconds'
endif

data.nvh(inorm)	=	mon			;monitor
data.mon	=	mon			;monitor
data.nvh(1)	=	col0			;collimations
data.nvh(2)	=	col1
data.nvh(3)	=	col2
data.nvh(4)	=	col3
data.nvh(5)	=	def
data.nvh(6)	=	dec
data.nvh(7)	=	dqc0
data.nvh(8)	=	dqc1
data.nvh(9)	=	dhfld
data.nvh(10)	=	msm
data.nvh(11)	=	mss
data.nvh(12)	=	msa
data.nvh(13)	=	taum
data.nvh(14)	=	taua
data.nvh(15)	=	dde
data.nvh(16)	=	ddq0
data.nvh(17)	=	ddq1
data.nvh(18)	=	ddq2
data.nvh(19)	=	dangle
data.nvh(20)	=	aa
data.nvh(21)	=	bb
data.nvh(22)	=	cc
data.nvh(23)	=	alfa
data.nvh(24)	=	beta
data.nvh(25)	=	gama

data.de(*,*)		=	dede(*,*)
data.asd(*,*)		=	asd(*,*)
data.dasd(*,*)		=	sqrt(asd(*,*))

;________________________________________________
;|						|
;|selecting the x,y axis			|
;|______________________________________________|
if n_elements(xtt) eq 0 then sxya,data
saxis,data,xtitle=xtt,ytitle=ytt

if data.tde(0) eq 'E' then begin
	data.tnvh(6)		=	'lc'
	data.unvh(6)		=	'rlu'
	data.nvh(6)		=	dqc2
endif else if data.tde(0) eq 'h' then begin
	data.tnvh(7)		=	'lc'
	data.unvh(7)		=	'rlu'
	data.nvh(7)		=	dqc2
endif else if data.tde(0) eq 'k' then begin
	data.tnvh(8)		=	'lc'
	data.unvh(8)		=	'rlu'
	data.nvh(8)		=	dqc2
endif
end



pro rasdq,filename,data,gaussian=gaussian,xtitle=xtt,ytitle=ytt,ct=ct
;----------------------------------------------------------------
;| Purpose	: read the data files with one Ei		|
;|			into data structure			|
;|								|
;| Input	:						|
;|	-- filename, string array for filenames			|
;|	-- xtt,	string for the x title				|
;|	-- ytt,	string for the y title				|
;| 								|
;| External	:						|
;|	-- rasdq.pro	&	combasd.pro  or subtasd.pro	|
;| 								|
;|
;| Modifications :						|
;|	Name modified 2/13/93 by Collin Broholm also		|
;|	keyword changed to gassian not gaussan			|
;----------------------------------------------------------------
common comwn,nwarno,nwartn
common comws,nwarts

nwartn		=	0
nwarts		=	0
nwarno		=	0
nfile		=	n_elements(filename)

	roneasdq,filename(0),data,xtitle=xtt,ytitle=ytt
	strdu,data
if nfile le 1 then goto,onefile
for i=1,nfile-1 do begin
	roneasdq,filename(i),datao,xtitle=xtt,ytitle=ytt
	strdu,datao
	merge,data,datao,d
	data	=	d
endfor


onefile:
end
